[![DOI](https://zenodo.org/badge/123999066.svg)](https://zenodo.org/badge/latestdoi/123999066)




[Description](#Desc) 
/ [Use](#Use) 
/ [Short Course Details](#course_details) 
/ [Citation](#Citation)

# Processing, modelling and vizualising hydrological data in R 

## <a name="Desc"></a> Description

This tutorial explores the use of some *tidyverse* approaches to develop a reproducible and scalable hydrological workflow. I investigate the role of catchment characteristics in controlling peak discharges. For this, I draw upon *piping*, *nesting* and *mapping* to apply a series of models and to identify the best fitting ones. Some custom functions are introduced to read in catchment descriptor data and to customize some output figures.

*Note: Data from the UK National River Flow Archive.
Please refer to http://nrfa.ceh.ac.uk/costs-terms-and-conditions prior to any re-use*

## <a name="Use"></a> Use

Files in this repository are intendeded to be used with the latest version of R (currently v3.5.2) as well as the latest version of RStudio (currently v1.1.463). 
To reproduce the presentation clone or download (and unzip) this repository to your local machine and open the project file **rHydro_2018.proj**. 
Navigate to and open the presentation file [**./presentation/01_rHydro_AGLHurley_proc-mod-viz.Rmd**](https://github.com/the-Hull/rHydro_2018/blob/master/presentation/01_rHydro_AGLHurley_proc-mod-viz.Rmd). 
Lastly, click the *knit* below the tabs bar in the RStudio IDE.

## <a name="course_details"></a> Using R in Hydrology - EGU 2018 Course Details 
Conveners: Louise Slater, Shaun Harrigan, Tobias Gauster, Alexander Hurley, Guillaume Thirel, Claudia Vitolo

This repository contains my a tutorial that I presented during the [short course at EGU 2018 in Vienna](#course_details). 
Other materials from this course can be found [here](https://github.com/hydrosoc/rhydro_EGU18) on the YHS GitHub page.


- **Where?** Wednesday 11 April at 10:30–12:00 in Room -2.16 (*basement!*); http://meetingorganizer.copernicus.org/EGU2018/session/28914
- **What?** This session is aimed at hydrologists who are interested in hearing more about R as well as those who are advanced R programmers wanting to discuss recent developments in an open environment. 
- The session is organised in cooperation with the <a href="https://younghs.com/" rel="nofollow">Young Hydrologic Society (YHS)</a>.
- Participants are invited to post and discuss questions in the
<a href="https://www.facebook.com/groups/1130214777123909/" rel="nofollow">Hydrology in R Facebook group</a>.



## <a name="Citation"></a> Citation

Following the [DOI](https://zenodo.org/badge/latestdoi/123999066), please refer to this work as:

* Alexander Hurley. (2019, January 31). Processing, modelling and vizualising hydrological data in R (Version 1.1). Zenodo. http://doi.org/10.5281/zenodo.2553980  
* Alexander Hurley, 2019. Processing, modelling and vizualising hydrological data in R. doi:10.5281/zenodo.2553980  


**BibTeX**  

```
@article{alexander hurley_2019, title={Processing, modelling and vizualising hydrological data in R}, DOI={10.5281/zenodo.2553980}, abstractNote={<p>This repository contains a all code, data and images to to reproduce the *.html* output of my <a href="https://meetingorganizer.copernicus.org/EGU2018/session/28914">short course contribution at EGU 2018</a>. Details can be found here: [the-Hull/rHydro_2018](https://github.com/the-Hull/rHydro_2018)</p>}, note={Note: Data from the UK National River Flow Archive. Please refer to http://nrfa.ceh.ac.uk/costs-terms-and-conditions prior to any re-use}, publisher={Zenodo}, author={Alexander Hurley}, year={2019}, month={Jan}}
```
